//! Elliott Wave module - Elliott wave patterns

pub mod elliott_correction;
pub mod elliott_double_combo;
pub mod elliott_impulse;
pub mod elliott_triangle;
pub mod elliott_triple_combo;

pub use elliott_correction::ElliottCorrection;
pub use elliott_double_combo::ElliottDoubleCombo;
pub use elliott_impulse::ElliottImpulse;
pub use elliott_triangle::ElliottTriangle;
pub use elliott_triple_combo::ElliottTripleCombo;
